/*
 * Asterisk -- A telephony toolkit for Linux.
 *
 * URI encoding / decoding
 * 
 * For now this code only supports 8 bit characters, not unicode,
 * which we ultimately will need to support.
 * 
 * Copyright (C) 2005, Digium, Inc.
 * Created by Olle E. Johansson, Edvina.net 
 *
 * This program is free software, distributed under the terms of
 * the GNU General Public License
 */

#include <stdlib.h>
#include <string.h>
#include <sys/types.h>

#include "asterisk.h"

/* ASTERISK_FILE_VERSION(__FILE__, "$Revision: 1.4 $") */

#include "asterisk/channel.h"
#include "asterisk/pbx.h"
#include "asterisk/logger.h"
#include "asterisk/utils.h"
#include "asterisk/app.h"

/*--- builtin_function_urlencode: Encode URL according to RFC 2396 */
static char *builtin_function_urlencode(struct ast_channel *chan, char *cmd, char *data, char *buf, size_t len) 
{
	char url[BUFSIZ];

	if (!data || ast_strlen_zero(data)) {
		ast_log(LOG_WARNING, "Syntax: URLENCODE(<data>) - missing argument!\n");
		return NULL;
	}

	url_encode(data, url, sizeof(url), 1);
	ast_copy_string(buf, url, len);

	return buf;
}

/*--- builtin_function_urldecode: Decode URL according to RFC 2396 */
static char *builtin_function_urldecode(struct ast_channel *chan, char *cmd, char *data, char *buf, size_t len) 
{
	if (!data || ast_strlen_zero(data)) {
		ast_log(LOG_WARNING, "Syntax: URLDECODE(<data>) - missing argument!\n");
		return NULL;
	}

	
	ast_copy_string(buf, data, len);
	url_decode(buf);
	return buf;
}

#ifndef BUILTIN_FUNC
static
#endif
struct ast_custom_function urldecode_function = {
	.name = "URLDECODE",
	.synopsis = "Decodes an URL-encoded string.",
	.syntax = "URLDECODE(<data>)",
	.read = builtin_function_urldecode,
};

#ifndef BUILTIN_FUNC
static
#endif
struct ast_custom_function urlencode_function = {
	.name = "URLENCODE",
	.synopsis = "Encodes a string to URL-safe encoding.",
	.syntax = "URLENCODE(<data>)",
	.read = builtin_function_urlencode,
};
